@@ -1,10 +1,11 @@
 /*
  * tools/lib/debug.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * June 1997
  * January 2000
+ * January,April 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -27,33 +28,40 @@
 /*
  * Changelog
  *
- *   30/01/2000 - extended va buffer
- *		- implemented debug_enter()/debug_leave()
+ *    30/01/2000 - extended va buffer
+ * 		- implemented debug_enter()/debug_leave()
+ *    23/01/2001 - changed to use lvm_log [JT]
+ *    19/02/2001 - changed names to have lvm_ prefix [JT]
+ *    09/04/2001 - moved var definition into if ( opt_d > 0)
  *
  */
 
 #ifdef DEBUG
-#include <liblvm.h>
+#include "liblvm.h"
 #define	BUFLEN	512
 static int call_depth = 0;
 
-void debug_out(char *buf)
-{
-   int i, c;
+static void _debug_out(char *message) {
+   char buffer[64], *ptr, c;
+   int i, n;
 
-   if ( call_depth > 0 && call_depth < BUFLEN - strlen(buf)) {
-      c = call_depth > 9 ? 'A' - 10 + call_depth : '0' + call_depth;
-      fputc ( '<', stderr);
-      for ( i = 0; i < call_depth; i++) fprintf ( stderr, "%c", c);
-      fprintf ( stderr, "> ");
-   }
-   fprintf ( stderr, buf);
-   fflush ( stderr);
-   return;
+   n = sizeof(buffer) - 4;
+   n = (call_depth < n) ? call_depth : n;
+   ptr = buffer;
+   c = call_depth > 9 ? 'A' - 10 + call_depth : '0' + call_depth;
+   *ptr++ = '<';
+
+   for(i = 0; i < n; i++)
+     *ptr++ = c;
+   *ptr++ = '>';
+   *ptr = '\0';
+
+   /* just using fatal here to ensure it's displayed */
+   print_log(LOG_FATAL, "%s %s", buffer, message);
 }
 
 
-void debug(const char *fmt, ...) {
+void lvm_debug(const char *fmt, ...) {
 
    if ( opt_d > 0) {
       va_list val;
@@ -61,42 +69,40 @@ void debug(const char *fmt, ...) {
 
       if ( call_depth < 0) call_depth = 0;
       va_start ( val, fmt);
-      vsprintf ( buf, fmt, val);
+      memset ( buf, 0, sizeof ( buf));
+      vsnprintf ( buf, sizeof ( buf) - 1, fmt, val);
       va_end ( val);
-      debug_out(buf);
+      _debug_out(buf);
    }
-   return;
 }
 
 
-void debug_enter(const char *fmt, ...)
-{
+void lvm_debug_enter(const char *fmt, ...) {
    if ( opt_d > 0) {
       va_list val;
       char buf[BUFLEN];
 
       call_depth++;
       va_start ( val, fmt);
-      vsprintf ( buf, fmt, val);
+      memset ( buf, 0, sizeof ( buf));
+      vsnprintf ( buf, sizeof ( buf) - 1, fmt, val);
       va_end ( val);
-      debug_out(buf);
+      _debug_out(buf);
    }
-   return;
 }
 
 
-void debug_leave(const char *fmt, ...)
-{
-   va_list val;
-   char buf[BUFLEN];
-
+void lvm_debug_leave(const char *fmt, ...) {
    if ( opt_d > 0) {
+      va_list val;
+      char buf[BUFLEN];
+
       va_start ( val, fmt);
-      vsprintf ( buf, fmt, val);
+      memset ( buf, 0, sizeof ( buf));
+      vsnprintf ( buf, sizeof ( buf) - 1, fmt, val);
       va_end ( val);
-      debug_out(buf);
+      _debug_out(buf);
       call_depth--;
    }
-   return;
 }
 #endif
