@@ -1,7 +1,7 @@
 /*
  * Tools/lib/liblvm.h
  *
- * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * March-June 1997
  * January 1998
@@ -42,6 +42,9 @@
  *                lvm_tab_check_free_vg_number and lvm_tab_get_all_vg_numbers
  *   23/10/2001 - added new lvmtab function lvm_tab_check_free_lv_numbers
  *   16/01/2002 - ATARAID support
+ *   29/02/2002 - removed wrong PV size maximum error code
+ *   11/02/2002 - changed pv_get_size() cast in order to support > 1TB volumes
+ *   17/02/2002 - added pv_get_size_ll() for lib compatibility
  *
  */
 
@@ -88,6 +91,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
+#include <limits.h>
 #ifdef _G_LSEEK64
 int lseek64 ( unsigned int, unsigned long long, unsigned int);
 #define llseek lseek64
@@ -99,6 +103,12 @@ int llseek ( unsigned int, unsigned long long, unsigned int);
 #include <linux/major.h>
 #include <linux/genhd.h>
 
+#ifdef major
+#  undef major
+#endif
+#ifdef minor
+#  undef minor
+#endif
 #include <lvm.h>
 
 #include "lvm_log.h"
@@ -153,6 +163,11 @@ extern struct config_file *config_file;
 
 #define	LVM_VGDA_SIZE(pv) ((pv)->pe_on_disk.base + (pv)->pe_on_disk.size)
 
+/* sector maximum based on max ~64k extents or 2TB in SECTORS of 512 byte */
+#define	LVM_LV_SIZE_2TB(a)	( ( long long) LVM_PE_T_MAX * (a)->pe_size > ( long long) 4*1024*1024*1024-1 ? ( long long) 4*1024*1024*1024-1 : ( long long) LVM_PE_T_MAX * (a)->pe_size)
+
+#define	LVM_PV_SIZE_MAX	( ( long long) (uint32_t) -1)
+
 #define	LVM_PE_ALIGN		 (65536UL / SECTOR_SIZE)
 
 
@@ -258,7 +273,6 @@ int    pv_change ( char *, pv_t *);
 int    pv_change_all_pv_of_vg ( char *, vg_t *);
 int    pv_change_all_pv_for_lv_of_vg ( char *, char *, vg_t *);
 int    pv_check_active ( char *, char *);
-int    pv_check_active_in_all_vg ( char *);
 int    pv_check_free ( pv_t *, ulong, ulong *);
 int    pv_check_free_contiguous ( pv_t *, ulong, ulong *);
 int    pv_check_in_vg ( vg_t *, char *);
@@ -271,15 +285,16 @@ int    pv_check_part ( char *);
 int    pv_check_volume ( char *, pv_t *);
 pv_t   *pv_copy_from_disk ( pv_disk_t *);
 pv_disk_t *pv_copy_to_disk ( pv_t *);
-char   **pv_find_all_pv_names ( void);
 kdev_t pv_create_kdev_t ( char *);
 char   *pv_create_name_from_kdev_t ( kdev_t);
+char   **pv_find_all_pv_names ( void);
 int    pv_find_vg ( char *, char **);
 int    pv_flush ( char *);
 int    pv_get_index_by_kdev_t ( vg_t *, kdev_t);
 int    pv_get_index_by_name ( vg_t *, char *);
 kdev_t pv_get_kdev_t_by_number ( vg_t *, int);
-long pv_get_size(char *, struct partition *);
+long  pv_get_size(char *, struct partition *);
+long long  pv_get_size_ll(char *, struct partition *);
 int    pv_move_pes ( vg_t*, char*, size_t, char**, int, int,
                      long*, long*, long*, int, int);
 int    pv_move_pe ( vg_t*, char*, size_t, long, long, long, long,
@@ -404,6 +419,7 @@ int    lvm_check_partitioned_dev ( dev_t);
 int    lvm_check_whole_disk_dev ( dev_t);
 void   lvm_check_special ( void);
 long   lvm_check_number ( char *, int);
+long long lvm_check_number_ll ( char *, int);
 int    lvm_check_uuid ( char*);
 unsigned char *lvm_create_uuid ( int);
 char   *lvm_show_uuid ( char *);
@@ -424,6 +440,7 @@ int    lvm_unlock ( void);
 /* LVMTAB based functions */
 int  lvm_tab_read ( char **, int *);
 int  lvm_tab_write ( char *, int);
+int  lvm_tab_check_free_lv_number ( lv_t *);
 int  lvm_tab_check_free_lv_numbers ( vg_t *);
 int  lvm_tab_check_free_vg_number ( vg_t *);
 int  lvm_tab_create ( void);
@@ -793,7 +810,7 @@ char **lvm_tab_vg_check_exist_all_vg ( void);
 #define LVM_EVG_WRITE_WRITE                                                 404
 #define LVM_ELV_PV_CREATE_NAME_FROM_KDEV_T                                  405
 #define LVM_EPV_FLUSH_STAT                                                  406
-#define LVM_EVG_SETUP_FOR_CREATE_PV_SIZE_MAX				    407
+#define LVM_MAX_ERROR							    407
 
 
 /*
