@@ -1,9 +1,10 @@
 /*
  * tools/lib/lv_read_COW_table.c
  *
- * Copyright (C) 2000  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 2000 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * July 2000
+ * February 2002
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -23,6 +24,13 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    17/02/2002 - remove fsync/pv_flush() because we don't write here
+ *
+ */
+
 #include <liblvm.h>
 
 
@@ -38,23 +46,28 @@ int lv_read_COW_table ( vg_t *vg, lv_t *lv) {
    kdev_t last_dev = 0;
    loff_t offset = 0;
    lv_COW_table_disk_t *lv_COW_table_disk = NULL;
+   char *pv_name = 0;
 
-#ifdef DEBUG
    debug_enter ( "lv_read_COW_table -- CALLED with: \"%s\" \"%s\"\n",
                  vg->vg_name, lv->lv_name);
-#endif
-   if ( vg == NULL ||
-        lv == NULL ||
-        vg_check_name ( vg->vg_name) < 0 ||
-        lv_check_name ( lv->lv_name) < 0) ret = -LVM_EPARAM;
-   else {
+
+   if ( vg == NULL || lv == NULL)
+      ret = -LVM_EPARAM;
+   else if ( ( ret = vg_check_name ( vg->vg_name)) == 0 &&
+             ( ret = lv_check_name ( lv->lv_name)) == 0) {
       end = LVM_GET_COW_TABLE_ENTRIES_PER_PE(vg, lv);
+
+      if(!end) {
+	      ret = -LVM_ELV_READ_COW_TABLE_MALLOC;
+	      goto out;
+      }
+
       COW_table_chunks_per_PE = LVM_GET_COW_TABLE_CHUNKS_PER_PE(vg, lv);
       size = end * sizeof ( lv_COW_table_disk_t);
       if ( ( lv_COW_table_disk = malloc ( size)) == NULL) {
          fprintf ( stderr, "%s -- ERROR: malloc at line %d\n\n", cmd, __LINE__);
          ret = -LVM_ELV_READ_COW_TABLE_MALLOC;
-         goto lv_read_COW_table_end;
+         goto out;
       }
       memset ( lv_COW_table_disk, 0, size);
 
@@ -64,24 +77,26 @@ int lv_read_COW_table ( vg_t *vg, lv_t *lv) {
             last_dev = lv->lv_current_pe[le].dev;
             if ( pv_handle != -1 && close ( pv_handle) < 0) {
                ret = -LVM_ELV_READ_COW_TABLE_CLOSE;
-               goto lv_read_COW_table_end;
+               goto out;
             }                 
 
-            if ( ( pv_handle = open ( pv_create_name_from_kdev_t ( lv->lv_current_pe[le].dev), O_RDONLY)) == -1) {
+            pv_name = pv_create_name_from_kdev_t ( last_dev);
+
+            if ( ( pv_handle = open ( pv_name, O_RDONLY)) == -1) {
                ret = -LVM_ELV_READ_COW_TABLE_OPEN;
-               goto lv_read_COW_table_end;
+               goto out;
             }                 
          }
 
          offset = ( loff_t) lv->lv_current_pe[le].pe * SECTOR_SIZE;
          if ( llseek ( pv_handle, offset, SEEK_SET) == -1) {
             ret = -LVM_ELV_READ_COW_TABLE_LLSEEK;
-            goto lv_read_COW_table_end;
+            goto out;
          }                 
 
          if ( read ( pv_handle, lv_COW_table_disk, size) != size) {
             ret = -LVM_ELV_READ_COW_TABLE_READ;
-            goto lv_read_COW_table_end;
+            goto out;
          }                 
 
          for ( e = 0; e < end; e++) {
@@ -103,15 +118,17 @@ int lv_read_COW_table ( vg_t *vg, lv_t *lv) {
       lv->lv_remap_ptr = index;
    }
 
-lv_read_COW_table_end:
-   if ( pv_handle != -1) close ( pv_handle);
+ out:
+   if ( pv_handle != -1) {
+      if ( close ( pv_handle) < 0) ret = -LVM_ELV_INIT_COW_TABLE_CLOSE;
+   }
+
    if ( ret < 0) {
       free ( lv->lv_block_exception);
       lv->lv_block_exception = NULL;
    }
    if ( lv_COW_table_disk != NULL) free ( lv_COW_table_disk);
-#ifdef DEBUG
+
    debug_leave ( "lv_read_COW_table -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
