@@ -1,12 +1,14 @@
 /*
  * tools/lib/pv_move.c
  *
- * Copyright (C) 1997, 1998, 1999  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * May  1998
  * July 1998
  * January,February,September,October 1999
  * January,February 2000
+ * March 2001
+ * February 2002
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -29,7 +31,7 @@
 /*
  * Changelog
  *
- *    29/04/1998 - changed to new pe_lock() calling convention
+ *    29/04/1998 - changed to new _pe_lock() calling convention
  *    14/05/1998 - implemented handling of striped logical volumes
  *    16/05/1998 - implemented handling of contiguous (optional striped)
  *                 logical volumes
@@ -39,7 +41,7 @@
  *                 to one destination physical volume
  *    20/01/1999 - used LVM_PE_DISK_OFFSET macro in pv_move_pe()
  *    06/02/1999 - fixed lv_current_le corection bug
- *    16/02/1999 - changed to new pe_lock()
+ *    16/02/1999 - changed to new _pe_lock()
  *    26/08/1999 - support for loop devices (Johannes Deisenhofer
  *                                           jo@hexmac.com)
  *    29/10/1999 - fixed possible free() bug
@@ -48,9 +50,16 @@
  *                 on a destination physical volume
  *    03/02/2000 - use debug_enter()/debug_leave()
  *    15/02/2000 - use lvm_error()
+ *    05/04/2001 - changed parameter list of pv_move_pes() and pv_move()
+ *                 to reflect new buffer_size parameter (HM)
+ *               - changed pv_move_pe() to copy buffer_size'd chunks rather
+ *                 than whole PEs in order to reduce memory pressure on systems
+ *                 with little RAM (HM)
+ *    13/02/2002 - support to optionally ignore read errors
  *
  */
 
+
 #include <liblvm.h>
 #ifdef __KERNEL__
 #   undef __KERNEL__
@@ -62,12 +71,39 @@
 #   define __KERNEL__
 #endif
 
-void pv_move_interrupt ( int);
+extern int opt_ignore;
+int vg_active = TRUE;
+
+/* internal functions */
+ulong  _read ( int, char*, ulong, char*);
+loff_t _llseek ( int, loff_t, int);
+void   pv_move_interrupt ( int);
+int   _pe_lock ( char*, kdev_t, ulong, ushort, ushort, kdev_t);
+int   _pe_unlock ( char*);
+int   _lv_le_remap ( vg_t*, le_remap_req_t*);
+int   _pv_change ( char*, pv_t*);
+
 static int pv_move_int = FALSE;
 static lv_t *lv_this = NULL;
 
+struct pe_struct {
+   ushort p;
+   long pe;
+   ushort lv_num;
+   ushort le_num;
+};
+
+int pe_compare(const void *one, const void *two)
+{
+   struct pe_struct *pe_one = (struct pe_struct *)one;
+   struct pe_struct *pe_two = (struct pe_struct *)two;
+
+   return pe_one->lv_num == pe_two->lv_num ? (pe_one->le_num - pe_two->le_num) :
+                                             (pe_one->lv_num - pe_two->lv_num);
+}
+
 /* perform the move of all physical extents / those of a logical volume */
-int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
+int pv_move_pes ( vg_t *vg, char *buffer, size_t buffer_size, char **pv_allowed,
                   int src_pv_index, int lv_index,
                   long *les, long *source_pes, long *dest_pes,
                   int opt_v, int opt_t) {
@@ -76,33 +112,26 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
    int dst_pv_index = 0;
    int source_pe_count = 0;
    int ext = 0;
-   int i = 0;
-   int l = 0;
    int pa = 0;
    int ret = 0;
    long le = 0;
    long pe = 0;
    long pe_for_dest = 0;
-   long pe_last = 0;
    long pe_to_move = 0;
    long pe_start = 0;
-   struct pe_struct {
-      ushort p;
-      long pe;
-      ushort lv_num;
-      ushort le_num;
-   } *pe_src = NULL, *pe_dest = NULL, *pe_src_sav, *pe_dest_sav, pe_tmp;
+   struct pe_struct *pe_src = NULL, *pe_src_sav;
+   struct pe_struct *pe_dest = NULL, *pe_dest_sav;
+   struct pe_struct pe_tmp;
 
-
-#ifdef DEBUG
    debug_enter ( "pv_move_pes -- CALLED\n");
-#endif
 
-   if ( vg == NULL || vg_check_consistency ( vg) < 0 ||
-        buffer == NULL || pv_allowed == NULL ||
+   if ( ( ret = vg_check_consistency ( vg)) < 0)
+      goto pv_move_pes_end;
+   if ( buffer == NULL || pv_allowed == NULL ||
         src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         lv_index < -1 || lv_index >= ( int) vg->lv_max ||
-        opt_v < 0 || opt_t < 0) {
+        opt_v < 0 || opt_t < 0 ||
+        buffer_size < SECTOR_SIZE || buffer_size % SECTOR_SIZE != 0) {
       ret = -LVM_EPARAM;
       goto pv_move_pes_end;
    }
@@ -148,6 +177,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
    }
    /* end of parameter check */
 
+   vg_active = vg_check_active ( vg->vg_name);
    pv_move_int = FALSE;
 
 
@@ -192,7 +222,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
          pe_src_sav = pe_src;
          if ( ( pe_src = realloc ( pe_src, ( pe_to_move + 1) *
                                    sizeof ( struct pe_struct))) == NULL) {
-            fprintf ( stderr, "%s -- realloc error in %s [line %d]\n\n",
+            fprintf ( stderr, "%s -- realloc error in %s [line %d]\n",
                               cmd, __FILE__, __LINE__);
             if ( pe_src_sav != NULL) free ( pe_src_sav);
             ret = -LVM_EPV_MOVE_PES_REALLOC;
@@ -212,33 +242,8 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
       goto pv_move_pes_end;
    }
 
-   /* sort source PEs in ascending LE order */
-   for ( l = 0; l < vg->pv[src_pv_index]->lv_cur; l++) {
-      for ( pe = pe_last ; pe < pe_to_move - 1; pe++) {
-         if ( pe_src[pe].lv_num > pe_src[pe+1].lv_num) {
-            memcpy ( &pe_tmp, &pe_src[pe], sizeof ( pe_tmp));
-            memcpy ( &pe_src[pe], &pe_src[pe+1], sizeof ( pe_tmp));
-            memcpy ( &pe_src[pe+1], &pe_tmp, sizeof ( pe_tmp));
-         }
-      }
-   }
-
-   pe_last = 0;
-   for ( i = 0; i < pe_to_move; i++) {
-      for ( pe = pe_last ; pe < pe_to_move - 1; pe++) {
-         if ( pe_src[pe].lv_num == pe_src[pe+1].lv_num) {
-            if ( pe_src[pe].le_num > pe_src[pe+1].le_num) {
-               memcpy ( &pe_tmp, &pe_src[pe], sizeof ( pe_tmp));
-               memcpy ( &pe_src[pe], &pe_src[pe+1], sizeof ( pe_tmp));
-               memcpy ( &pe_src[pe+1], &pe_tmp, sizeof ( pe_tmp));
-            }
-         } else {
-            pe_last = pe;
-            break;
-         }
-      }
-   }
-   /* END sort source PEs in ascending LE order */
+   /* sort source PEs in ascending LV, LE order */
+   qsort ( pe_src, pe_to_move, sizeof ( pe_tmp), pe_compare);
 
    /* get all destination PEs */
    pe_for_dest = 0;
@@ -290,7 +295,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
          pe_dest_sav = pe_dest;
          if ( ( pe_dest = realloc ( pe_dest, ( pe_for_dest + 1) *
                                     sizeof ( struct pe_struct))) == NULL) {
-            fprintf ( stderr, "%s -- realloc error in %s [line %d]\n\n",
+            fprintf ( stderr, "%s -- realloc error in %s [line %d]\n",
                               cmd, __FILE__, __LINE__);
             if ( pe_dest_sav != NULL) free ( pe_dest_sav);
             ret = -LVM_EPV_MOVE_PES_REALLOC;
@@ -323,14 +328,11 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
    for ( pe = 0; pv_move_int == FALSE && pe < pe_to_move; pe++) {
       lv_index = pe_src[pe].lv_num - 1;
       lv_this = vg->lv[lv_index];
-      if ( ( ret = pv_move_pe ( vg, buffer,
+      if ( ( ret = pv_move_pe ( vg, buffer, buffer_size,
                                 pe_src[pe].p,  pe_dest[pe].p,
                                 pe_src[pe].pe, pe_dest[pe].pe,
-                                opt_v, opt_t, pe + 1, pe_to_move)) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" pv_move_pe\n\n",
-                           cmd, lvm_error ( ret));
+                                opt_v, opt_t, pe + 1, pe_to_move)) < 0)
          goto pv_move_pes_end;
-      }
    }
 
 pv_move_pes_end:
@@ -339,16 +341,13 @@ pv_move_pes_end:
 
    if ( ret >= 0) ret = pe;
 
-#ifdef DEBUG
    debug_leave ( "pv_move_pes -- LEAVING with ret: %d\n", ret);
-#endif
-
    return ret;
 } /* pv_move_pes() */
 
 
 /* perform the move of a physical extent */
-int pv_move_pe ( vg_t *vg, char *buffer,
+int pv_move_pe ( vg_t *vg, char *buffer, size_t buffer_size,
                  long src_pv_index, long dst_pv_index,
                  long pe_source,    long pe_dest,
                  int opt_v, int opt_t,
@@ -364,22 +363,20 @@ int pv_move_pe ( vg_t *vg, char *buffer,
    size_t size = 0;
    le_remap_req_t le_remap_req;
 
-#ifdef DEBUG
    debug_enter ( "pv_move_pe -- CALLED\n");
-#endif
 
    if ( src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         dst_pv_index < 0 || dst_pv_index >= vg->pv_cur ||
         pe_source < 0 || pe_source >= vg->pv[src_pv_index]->pe_total ||
         pe_dest   < 0 || pe_dest   >= vg->pv[dst_pv_index]->pe_total ||
-        opt_v < 0 || opt_t < 0 || act_pe < 0 || off_pe < 0 ||
+        opt_t < 0 || act_pe < 0 || off_pe < 0 ||
         vg->pv[dst_pv_index]->pe[pe_dest].lv_num != 0) {
       ret = -LVM_EPARAM;
       goto pv_move_pe_end;
    }
 
    if ( ( in = open ( vg->pv[src_pv_index]->pv_name, O_RDONLY)) == -1) {
-      fprintf ( stderr, "%s -- couldn't open input physical volume %s\n\n",
+      fprintf ( stderr, "%s -- couldn't open input physical volume %s\n",
                 cmd, vg->pv[src_pv_index]->pv_name);
       ret = -LVM_EPV_MOVE_PE_OPEN_IN;
       goto pv_move_pe_end;
@@ -415,11 +412,13 @@ int pv_move_pe ( vg_t *vg, char *buffer,
    }
 
    /* Fill the remapping request structure */
-   strcpy ( le_remap_req.lv_name, lv_name_this);
+   memset ( &le_remap_req, 0, sizeof ( le_remap_req));
+   strncpy ( le_remap_req.lv_name, lv_name_this,
+             sizeof ( le_remap_req.lv_name) - 1);
    le_remap_req.old_dev = vg->pv[src_pv_index]->pv_dev;
    le_remap_req.new_dev = vg->pv[dst_pv_index]->pv_dev;
-   le_remap_req.old_pe  = LVM_PE_DISK_OFFSET( pe_source, vg->pv[src_pv_index]);
-   le_remap_req.new_pe  = LVM_PE_DISK_OFFSET( pe_dest, vg->pv[dst_pv_index]);
+   le_remap_req.old_pe  = get_pe_offset(pe_source, vg->pv[src_pv_index]);
+   le_remap_req.new_pe  = get_pe_offset(pe_dest, vg->pv[dst_pv_index]);
 
    /* remap lv_current_pe device and PE in arrays */
    for ( l = 0; l < vg->lv_max; l++) {
@@ -437,7 +436,7 @@ pv_move_pe_continue:
 
    if ( opt_v > 1) {
       printf ( "lv: %s[%d]  old_dev: %02d:%02d  new_dev: %02d:%02d  "
-               "old_pe_sector: %lu  new_pe_sector: %lu\n",
+               "old_pe_sector: %u  new_pe_sector: %u\n",
                le_remap_req.lv_name,
                vg->pv[dst_pv_index]->pe[pe_dest].lv_num,
                MAJOR ( le_remap_req.old_dev),
@@ -453,7 +452,7 @@ pv_move_pe_continue:
    if ( ( out = open ( vg->pv[dst_pv_index]->pv_name,
                        O_WRONLY)) == -1) {
       fprintf ( stderr, "%s -- couldn't open output "
-                        "physical volume \"%s\"\n\n",
+                        "physical volume \"%s\"\n",
                 cmd, vg->pv[dst_pv_index]->pv_name);
       ret = -LVM_EPV_MOVE_PE_OPEN;
       goto pv_move_pe_end;
@@ -462,27 +461,24 @@ pv_move_pe_continue:
    if ( opt_v > 1) printf ( "%s -- llseeking input physical volume \"%s\"\n",
                             cmd, vg->pv[src_pv_index]->pv_name);
    offset = ( loff_t) le_remap_req.old_pe * SECTOR_SIZE;
-   if ( ( result = llseek ( in, offset, SEEK_SET)) == -1) {
-      fprintf ( stderr, "%s -- couldn't llseek to %lu:%lu on input "
-                        "physical volume \"%s\"\n\n",
+   if ( ( result = _llseek ( in, offset, SEEK_SET)) == -1) {
+      fprintf ( stderr, "%s -- couldn't llseek to sector %u on input "
+                        "physical volume \"%s\"\n",
                 cmd,
-                ( long) ( offset >> 32),
-                ( long) ( offset & 0xFFFFFFFF),
+                le_remap_req.old_pe,
                 vg->pv[src_pv_index]->pv_name);
       ret = -LVM_EPV_MOVE_PE_LLSEEK_IN;
       goto pv_move_pe_end;
    }
 
-   if ( opt_v > 1) printf ( "%s -- llseeking output physical "
-                            "volume \"%s\"\n",
+   if ( opt_v > 1) printf ( "%s -- llseeking output physical volume \"%s\"\n",
                             cmd, vg->pv[dst_pv_index]->pv_name);
    offset = ( loff_t) le_remap_req.new_pe * SECTOR_SIZE;
    if ( ( result = llseek ( out, offset, SEEK_SET)) == -1) {
-      fprintf ( stderr, "%s -- couldn't llseek to %lu:%lu on output "
-                        "physical volume \"%s\"\n\n",
+      fprintf ( stderr, "%s -- couldn't llseek to sector %u on output "
+                        "physical volume \"%s\"\n",
                 cmd,
-                ( ulong) ( offset >> 32),
-                ( ulong) ( offset & 0xFFFFFFFF),
+                le_remap_req.new_pe,
                 vg->pv[dst_pv_index]->pv_name);
       ret = -LVM_EPV_MOVE_PE_LLSEEK_OUT;
       goto pv_move_pe_end;
@@ -508,65 +504,76 @@ pv_move_pe_continue:
                             cmd, pe_source,
                             vg->pv[src_pv_index]->pv_name);
    if ( opt_t == 0) {
-      if ( ( ret = pe_lock ( vg->vg_name,
-                             vg->pv[src_pv_index]->pv_dev,
-                             le_remap_req.old_pe,
-                             vg->vg_number,
-                             vg->pv[dst_pv_index]->\
-                             pe[pe_dest].lv_num,
-                             vg->lv[vg->pv[dst_pv_index]->\
-                             pe[pe_dest].lv_num-1]->lv_dev)) < 0) {
+      int lv_num = vg->pv[dst_pv_index]->pe[pe_dest].lv_num;
+
+      if ( ( ret = _pe_lock ( vg->vg_name, vg->pv[src_pv_index]->pv_dev,
+                             le_remap_req.old_pe, vg->vg_number,
+                             lv_num, vg->lv[lv_num-1]->lv_dev)) < 0) {
          ret = -LVM_EPV_MOVE_PE_LOCK;
          goto pv_move_pe_end;
       }
    }
 
-   /* Future HM extension: write time stamp for pe_lock o.k. */
 
-   if ( opt_v > 1) printf ( "%s -- reading input physical volume \"%s\"\n",
-                            cmd, vg->pv[src_pv_index]->pv_name);
+   /* we are going to read and write a whole physical extent
+      in buffer_size chunks maximum io requests */
    size = vg->pe_size * SECTOR_SIZE;
-   if ( opt_t == 0) {
-      if ( read ( in, buffer, size) != size) {
-         fprintf ( stderr, "%s -- ERROR reading input "
-                           "physical volume \"%s\"\n\n",
-                   cmd, vg->pv[src_pv_index]->pv_name);
-         pe_unlock ( vg->vg_name);
-         ret = -LVM_EPV_MOVE_PE_READ_IN;
-         goto pv_move_pe_end;
+
+   if ( opt_v > 1) printf ( "%s -- about to read input physical volume \"%s\" "
+                            "and to write output physical volume \"%s\"\n",
+                            cmd, vg->pv[src_pv_index]->pv_name,
+                            vg->pv[dst_pv_index]->pv_name);
+   /* we do it buffer_size chunk per chunk in order
+      to minimize memory pressure unless we find an error and opt_ignore is set
+      where we drop to SECTOR_SIZEed reads */
+   while ( size > 0) {
+      ulong red, to_read, to_write;
+
+      red = to_read = to_write = size > buffer_size ? buffer_size : size;
+      if ( opt_t == 0) {
+         if ( ( red = _read ( in, buffer, to_read,
+                              vg->pv[src_pv_index]->pv_name)) != to_read) {
+            fprintf ( stderr, "%s -- ERROR reading input "
+                              "physical volume \"%s\" (still %d bytes "
+                              "to read)\n",
+                      cmd, vg->pv[src_pv_index]->pv_name, size);
+            _pe_unlock ( vg->vg_name);
+            ret = -LVM_EPV_MOVE_PE_READ_IN;
+            goto pv_move_pe_end;
+         }
+         to_write = red;
       }
+   
+      if ( opt_t == 0) {
+         if ( write ( out, buffer, to_write) != to_write) {
+            fprintf ( stderr, "%s -- ERROR writing output "
+                              "physical volume \"%s\" (still %d bytes to "
+                              "write)\n",
+                              cmd, vg->pv[dst_pv_index]->pv_name, size);
+            _pe_unlock ( vg->vg_name);
+            ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
+            goto pv_move_pe_end;
+         }
+      }
+      size -= red;
    }
 
-   if ( opt_v > 1) printf ( "%s -- writing output physical volume \"%s\"\n",
-                            cmd, vg->pv[dst_pv_index]->pv_name);
-   if ( opt_t == 0) {
-      if ( write ( out, buffer, size) != size) {
-         fprintf ( stderr, "%s -- ERROR writing output "
-                           "physical volume \"%s\"\n\n",
-                   cmd, vg->pv[dst_pv_index]->pv_name);
-         pe_unlock ( vg->vg_name);
-         ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
-         goto pv_move_pe_end;
-      }
-      if ( fsync ( out) < 0  && (errno != EINVAL && errno != EROFS)) {
-         fprintf ( stderr, "%s -- ERROR syncing output to "
-                           "physical volume \"%s\"\n\n",
-                   cmd, vg->pv[dst_pv_index]->pv_name);
-         pe_unlock ( vg->vg_name);
-         ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
-         goto pv_move_pe_end;
-      }
+   if ( fsync ( out) < 0  && (errno != EINVAL && errno != EROFS)) {
+	   fprintf ( stderr, "%s -- ERROR syncing output to "
+		     "physical volume \"%s\"\n",
+		     cmd, vg->pv[dst_pv_index]->pv_name);
+	   _pe_unlock ( vg->vg_name);
+	   ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
+	   goto pv_move_pe_end;
    }
 
-   /* Future HM extension: write time stamp for lv_le_remap */
-
    if ( opt_v > 1) printf ( "%s -- remapping physical extent "
                             "in VGDA of kernel\n", cmd);
    if ( opt_t == 0) {
-      if ( ( ret = lv_le_remap ( vg, &le_remap_req)) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" remapping\n\n",
+      if ( ( ret = _lv_le_remap ( vg, &le_remap_req)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" remapping\n",
                            cmd, lvm_error ( ret));
-         pe_unlock ( vg->vg_name);
+         _pe_unlock ( vg->vg_name);
          ret = -LVM_EPV_MOVE_LV_LE_REMAP;
          goto pv_move_pe_end;
       }
@@ -575,8 +582,8 @@ pv_move_pe_continue:
    if ( opt_v > 1) printf ( "%s -- unlocking physical extent\n",
                             cmd);
    if ( opt_t == 0) {
-      if ( ( ret = pe_unlock ( vg->vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" unlocking PE %lu on \"%s\"\n\n",
+      if ( ( ret = _pe_unlock ( vg->vg_name)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" unlocking PE %lu on \"%s\"\n",
                            cmd, lvm_error ( ret),
                            pe_source,
                            vg->pv[src_pv_index]->pv_name);
@@ -589,8 +596,8 @@ pv_move_pe_continue:
                             "of kernel\n",
                             cmd, vg->pv[src_pv_index]->pv_name);
    if ( opt_t == 0) {
-      if ( ( ret = pv_change ( vg->vg_name, vg->pv[src_pv_index])) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" changing source \"%s\"\n\n",
+      if ( ( ret = _pv_change ( vg->vg_name, vg->pv[src_pv_index])) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" changing source \"%s\"\n",
                            cmd, lvm_error ( ret),
                            vg->pv[src_pv_index]->pv_name);
          ret = -LVM_EPV_MOVE_PV_CHANGE_SRC;
@@ -602,8 +609,8 @@ pv_move_pe_continue:
                             "of kernel\n",
                             cmd, vg->pv[dst_pv_index]->pv_name);
    if ( opt_t == 0) {
-      if ( ( ret = pv_change ( vg->vg_name, vg->pv[dst_pv_index])) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" changing destinatiom \"%s\"\n\n",
+      if ( ( ret = _pv_change ( vg->vg_name, vg->pv[dst_pv_index])) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" changing destinatiom \"%s\"\n",
                            cmd, lvm_error ( ret),
                            vg->pv[dst_pv_index]->pv_name);
          ret = -LVM_EPV_MOVE_PV_CHANGE_DEST;
@@ -618,7 +625,7 @@ pv_move_pe_continue:
       if ( ( ret = pv_write_with_pe ( vg->pv[src_pv_index]->pv_name,
                                       vg->pv[src_pv_index])) < 0) {
          fprintf ( stderr, "%s -- ERROR \"%s\" writing source "
-                           "physical volume data on \"%s\"\n\n", 
+                           "physical volume data on \"%s\"\n", 
                            cmd, lvm_error ( ret),
                            vg->pv[src_pv_index]->pv_name);
          ret = -LVM_EPV_MOVE_PV_PV_WRITE_WITH_PE_SRC;
@@ -633,7 +640,7 @@ pv_move_pe_continue:
       if ( ( ret = pv_write_with_pe ( vg->pv[dst_pv_index]->pv_name,
                                       vg->pv[dst_pv_index])) < 0) {
          fprintf ( stderr, "%s -- ERROR \"%s\" writing destination "
-                           "physical volume data on \"%s\"\n\n", 
+                           "physical volume data on \"%s\"\n", 
                            cmd, lvm_error ( ret),
                            vg->pv[dst_pv_index]->pv_name);
          ret = -LVM_EPV_MOVE_PV_PV_WRITE_WITH_PE_DEST;
@@ -645,9 +652,7 @@ pv_move_pe_end:
    if ( in != -1) close ( in);
    if ( out != -1) close ( out);
 
-#ifdef DEBUG
    debug_leave ( "pv_move_pe -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 } /* pv_move_pe() */
 
@@ -655,9 +660,8 @@ pv_move_pe_end:
 void pv_move_interrupt ( int sig) {
    static int first = 0;
 
-#ifdef DEBUG
    debug_enter ( "pv_move_interrupt -- CALLED\n");
-#endif
+
    signal ( SIGINT, pv_move_interrupt);
    if ( first != 0) {
       if ( lv_this->lv_stripes > 1) {
@@ -674,8 +678,86 @@ void pv_move_interrupt ( int sig) {
    } else first++;
 
 pv_move_interrupt_end:
-#ifdef DEBUG
+
    debug_leave ( "pv_move_interrupt -- LEAVING\n");
-#endif
    return;
 }
+
+/* llseek with optional error ignore */
+loff_t _llseek ( int fh, loff_t offset, int whence) {
+   loff_t pos = llseek ( fh, offset, whence);
+
+   if ( pos == ( loff_t) -1) {
+      if ( opt_ignore > 0) pos = 0;
+   }
+   return pos;
+}
+
+/* read with fallback to SECTOR_SIZEed io in case of an error */
+ulong _read ( int in, char *buffer, ulong to_read, char *pv_name) {
+   ulong ret;
+   ulong total_read = 0;
+   loff_t pos = _llseek ( in, 0, SEEK_CUR);
+
+   memset ( buffer, 0, to_read);
+
+   /* if we can't even seek */
+   if ( pos <= 0) {
+      if ( opt_ignore == 0) return 0;
+      return to_read;
+   }
+
+   if ( ( ret = read ( in, buffer, to_read)) != to_read) {
+      /* in case we shouldn't ignore, that's an error */
+      if ( opt_ignore == 0) return ret;
+
+      /* redo with SECTOR_SIZE reads */
+      memset ( buffer, 0, to_read);
+      if ( _llseek ( in, pos, SEEK_SET) <= 0) return to_read;
+      while ( total_read < to_read) {
+         if ( ( ret = read ( in, buffer, SECTOR_SIZE)) != SECTOR_SIZE) {
+            fprintf ( stderr, "%s -- ERROR \"%s\" reading sector %Ld from "
+                              "\"%s\"\n",
+                              cmd, strerror ( errno),
+                              pos / SECTOR_SIZE, pv_name);
+          }
+          pos += SECTOR_SIZE;
+          total_read += SECTOR_SIZE;
+          if ( _llseek ( in, pos, SEEK_SET) <= 0) return total_read;
+      }
+   }
+   return ret;
+} /* _read() */
+
+/* avoid pe_lock() in inactive VG */
+int _pe_lock ( char *vg_name, kdev_t pv_dev, ulong old_pe, ushort vg_number,
+               ushort lv_number, kdev_t lv_dev) {
+   if ( vg_active == TRUE)
+      return pe_lock ( vg_name, pv_dev, old_pe, vg_number, lv_number, lv_dev);
+   else
+      return 0;
+}
+
+/* avoid pe_unlock() in inactive VG */
+int _pe_unlock ( char *vg_name) {
+   if ( vg_active == TRUE)
+      return pe_unlock ( vg_name);
+   else
+      return 0;
+}
+
+/* avoid lv_le_remap() in inactive VG */
+int _lv_le_remap ( vg_t *vg, le_remap_req_t *le_remap_req) {
+   if ( vg_active == TRUE)
+      return lv_le_remap ( vg, le_remap_req);
+   else
+      return 0;
+}
+
+/* avoid pv_change() in inactive VG */
+int _pv_change ( char *vg_name, pv_t *pv) {
+   if ( vg_active == TRUE)
+      return pv_change ( vg_name, pv);
+   else
+      return 0;
+}
