@@ -1,10 +1,11 @@
 /*
  * tools/lib/vg_extend_reduce.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * April 1997
  * January 1999
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -28,6 +29,7 @@
  * Changelog
  *
  *    26/01/1999 - made volume group directory prefix a preprocessor option
+ *    08/02/2000 - use debug_enter()/debug_leave()
  *
  */
 
@@ -37,12 +39,12 @@
 int vg_extend_reduce ( char *, pv_t *, vg_t *, int);
 
 
-inline int vg_extend ( char *vg_name, pv_t *pv, vg_t *vg) {
+int vg_extend ( char *vg_name, pv_t *pv, vg_t *vg) {
    return vg_extend_reduce ( vg_name, pv, vg, VG_EXTEND);
 }
 
 
-inline int vg_reduce ( char *vg_name, pv_t *pv, vg_t *vg) {
+int vg_reduce ( char *vg_name, pv_t *pv, vg_t *vg) {
    return vg_extend_reduce ( vg_name, pv, vg, VG_REDUCE);
 }
 
@@ -52,38 +54,33 @@ int vg_extend_reduce ( char *vg_name, pv_t *pv, vg_t *vg, int cr) {
    int ret = 0;
    char group_file[NAME_LEN];
 
-#ifdef DEBUG
-   debug ( "vg_extend_reduce -- CALLED\n");
-#endif
+   debug_enter ( "vg_extend_reduce -- CALLED\n");
 
    if ( vg_name == NULL ||
         vg_check_name ( vg_name) < 0 ||
         pv == NULL ||
         vg == NULL ||
         pv_check_consistency ( pv) < 0 ||
-        vg_check_consistency_with_pv_and_lv ( vg) < 0) return -LVM_EPARAM;
-
-   sprintf ( group_file, LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
-   if ( ( group = open ( group_file, O_RDWR)) == -1)
-      ret = -LVM_EVG_EXTEND_REDUCE_OPEN;
+        vg_check_consistency_with_pv_and_lv ( vg) < 0) ret = -LVM_EPARAM;
    else {
-#ifdef DEBUG
-      debug ( "vg_extend_reduce -- IOCTL\n");
-#endif
-      if ( cr == VG_EXTEND) ret = ioctl ( group, cr, pv);
-      else                  ret = ioctl ( group, cr, pv->pv_name);
-      if ( ret == -1) ret = -errno;
+      memset ( group_file, 0, sizeof ( group_file));
+      snprintf ( group_file, sizeof ( group_file) - 1,
+                             LVM_DIR_PREFIX "%s/group%c", vg_name, 0);
+      if ( ( group = open ( group_file, O_RDWR)) == -1)
+         ret = -LVM_EVG_EXTEND_REDUCE_OPEN;
+      else {
+         debug ( "vg_extend_reduce -- IOCTL\n");
+         if ( cr == VG_EXTEND) ret = ioctl ( group, cr, pv);
+         else                  ret = ioctl ( group, cr, pv->pv_name);
+         if ( ret == -1) ret = -errno;
+      }
+   
+      if ( group != -1) close ( group);
+      debug ( "vg_extend_reduce -- IOCTL returned: %d\n", ret);
+      if ( ret == 0 && cr == VG_REDUCE)
+         ret = pv_change_all_pv_of_vg ( vg_name, vg);
    }
 
-   if ( group != -1) close ( group);
-#ifdef DEBUG
-   debug ( "vg_extend_reduce -- IOCTL returned: %d\n", ret);
-#endif
-   if ( ret == 0 && cr == VG_REDUCE)
-      ret = pv_change_all_pv_of_vg ( vg_name, vg);
-
-#ifdef DEBUG
-   debug ( "vg_extend_reduce -- LEAVING with ret: %d\n", ret);
-#endif
+   debug_leave ( "vg_extend_reduce -- LEAVING with ret: %d\n", ret);
    return ret;
 }
