@@ -126,17 +126,14 @@ int main ( int argc, char **argv) {
    while ( ( c = getopt_long ( argc, argv, options,
                                long_options, NULL)) != EOF) {
       switch ( c) {
+	 DEBUG_HANDLE_CASE_D;
+
          case 'a':
             opt_a++;
             opt++;
             opt_E++; opt_L++; opt_U++; opt_P++; opt_V++;
             break;
 
-         case 'd':
-            opt_d++;
-            opt++;
-            break;
-
          case 'E':
             opt_E++;
             opt++;
