@@ -1,13 +1,14 @@
 /*
  * tools/pvmove.c
  *
- * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * June,October 1997
  * April,May,July,September 1998
  * January,October 1999
  * February 2000
  * January,February 2001
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +46,7 @@
  *    20/02/2001 - fixed bug preventing move from/to MD devices by checking
  *                 LVM_EPV_READ_MD_DEVICE pv_read() return code (Andreas Dilger)
  *    19/03/2001 - Fix some typos in messages
+ *    13/02/2002 - implemented -i option
  *
  */
 
@@ -69,9 +71,10 @@ void usage(int ret)
    fprintf(stream, "\n%s "
            "[-A|--autobackup {y|n}] "
            DEBUG_HELP_2
-           "[-f|--force]"
-           "[-h|--help]\n\t"
-           "[-t|--test] "
+           "[-f|--force]\n\t"
+           "[-i|--ignore_read_errors] "
+           "[-h|--help] "
+           "[-t|--test]\n\t"
            "[-v[v]|--verbose [--verbose]] "
            "[--version]\n\t"
            "[{-n|--name} LogicalVolume[:LogicalExtent[-LogicalExtent]...]]\n\t"
@@ -100,7 +103,6 @@ int main ( int argc, char **argv) {
    int ret = 0;
    int dst_pv_index = 0;
    int src_pv_index = 0;
-   int vg_active = FALSE;
    long *les = NULL;
    long *source_pes = NULL;
    long *dest_pes = NULL;
@@ -108,16 +110,17 @@ int main ( int argc, char **argv) {
    char lv_name_buf[NAME_LEN] = { 0, };
    char *buffer = NULL;
    char *lv_name = NULL;
-   char *options = "A:fhn:tv" DEBUG_SHORT_OPTION;
+   char *options = "A:fhin:tv" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
       { "autobackup", required_argument, NULL, 'A'},
       DEBUG_LONG_OPTION
-      { "force",      no_argument,       NULL, 'f'},
-      { "help",       no_argument,       NULL, 'h'},
-      { "name",       required_argument, NULL, 'n'},
-      { "test",       no_argument,       NULL, 't'},
-      { "verbose",    no_argument,       NULL, 'v'},
-      { "version",    no_argument,       NULL, 22 },
+      { "force",              no_argument,       NULL, 'f'},
+      { "help",               no_argument,       NULL, 'h'},
+      { "ignore_read_errors", no_argument,       NULL, 'i'},
+      { "name",               required_argument, NULL, 'n'},
+      { "test",               no_argument,       NULL, 't'},
+      { "verbose",            no_argument,       NULL, 'v'},
+      { "version",            no_argument,       NULL, 22 },
       { NULL, 0, NULL, 0}
    };
    char **pv_allowed = NULL;
@@ -156,6 +159,10 @@ int main ( int argc, char **argv) {
             usage(0);
             break;
 
+         case 'i':
+            opt_ignore++;
+            break;
+
          case 'n':
             lv_name = optarg;
             if ( lvm_get_col_numbers ( optarg, &les) < 0)
@@ -301,7 +308,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\"\n",
                                cmd, lv_name);
       if ( strchr ( lv_name, '/') == NULL) {
-         sprintf ( lv_name_buf, LVM_DIR_PREFIX "%s/%s", vg_name, lv_name);
+         memset ( lv_name_buf, 0, sizeof ( lv_name_buf));
+         snprintf ( lv_name_buf, sizeof ( lv_name_buf) - 1,
+                                 LVM_DIR_PREFIX "%s/%s", vg_name, lv_name);
          lv_name = lv_name_buf;
       }
       if ( lv_check_name ( lv_name) < 0) {
@@ -461,14 +470,10 @@ int main ( int argc, char **argv) {
    }
    
    if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
-   printf ( "%s -- moving physical extents in ", cmd);
-   if ( vg_check_active ( vg_name) == TRUE) {
-      vg_active = TRUE;
-   } else {
-      vg_active = FALSE;
-      printf ( "in");
-   }
-   printf ( "active volume group \"%s\"\n", vg_name);
+   printf ( "%s -- moving physical extents in %sactive volume group \"%s\"\n",
+            cmd,
+            vg_check_active ( vg_name) == TRUE ? "" : "in",
+            vg_name);
 
    if ( opt_f == 0) {
       printf ( "%s -- WARNING: if you lose power during the move you may need\n"
