@@ -392,16 +392,26 @@ int main ( int argc, char **argv) {
       return LVM_EVGIMPORT_NO_DEV;
    }
 
-   if ( opt_v > 0)
-      printf ( "%s -- importing logical volumes of volume group \"%s\"\n",
-               cmd, vg->vg_name);
    for ( blk_dev = l = 0; l < vg->lv_max; l++) {
-      if ( vg->lv[l] != NULL) {
-         strcpy ( vg->lv[l]->vg_name, vg_name);
-         strcpy ( vg->lv[l]->lv_name,
-                  lv_change_vgname ( vg_name, vg->lv[l]->lv_name));
-         vg->lv[l]->lv_dev = blk_dev_free[blk_dev++];
+      lv_t *lv = vg->lv[l];
+      if ( lv == NULL) continue;
+      strcpy ( lv->vg_name, vg_name);
+      strcpy ( lv->lv_name,
+               lv_change_vgname ( vg_name, lv->lv_name));
+      if ( lv->lv_access & LV_SNAPSHOT_ORG) {
+         int ll;
+         for ( ll = 0; ll < vg->lv_max; ll++) {
+            lv_t *lv1 = vg->lv[ll];
+            if ( lv1 == NULL || lv1 == lv) continue;
+            if ( lv1->lv_access & LV_SNAPSHOT &&
+                 lv1->lv_snapshot_minor == MINOR ( lv->lv_dev)) {
+               printf ( "%s -- correcting snapshot relationship of \"%s\"\n",
+                        cmd, lv1->lv_name);
+               lv1->lv_snapshot_minor = MINOR( blk_dev_free[blk_dev]);
+            }
+         }
       }
+      lv->lv_dev = blk_dev_free[blk_dev++];
    }
 
    /* correct status to import */
