@@ -1,10 +1,11 @@
 /*
  * tools/vgmknodes.c
  *
- * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Sistina Software
  *
  * May-June,September 1998
- * January 1999
+ * January,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -29,6 +30,9 @@
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    05/09/1998 - corrected some messages
  *    26/01/1999 - made volume group directory prefix a preprocessor option
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
  *
  */
 
@@ -46,44 +50,37 @@ int main ( int argc, char **argv)
    int length = 0;
    int opt_v = 0;
    int ret = 0;
-#ifdef DEBUG
-   char *options = "dh?v";
-#else
-   char *options = "h?v";
-#endif
+   char *options = "h?v" DEBUG_SHORT_OPTION;
+   struct option long_options[] = {
+      DEBUG_LONG_OPTION
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL,      0,           NULL, 0},
+   };
    char *vg_name = NULL;
    vg_t *vg = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return 1;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Make Nodes\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Make Nodes\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-#ifdef DEBUG
-                     "\t[-d]\n"
-#endif
-                     "\t[-h/-?]\n"
-                     "\t[-v]\n"
+                     DEBUG_HELP
+                     "\t[-h/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\t[VolumeGroupName...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return LVM_EINVALID_CMD_LINE;
             break;
 
@@ -141,16 +138,16 @@ int main ( int argc, char **argv)
                                "volume group \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_remove_dir_and_group_and_nodes ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: removing volume group "
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group "
                            "nodes of \"%s\"\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          return LVM_EVGMKNODES_VG_REMOVE_DIR_AND_GROUP_NODES;
       }
    
       if ( ( ret = vg_create_dir_and_group_and_nodes ( vg, opt_v)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: creating volume group "
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                            "nodes of \"%s\"\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          return LVM_EVGMKNODES_VG_CREATE_DIR_AND_GROUP_NODES;
       }
    
